SUBROUTINE SurfBoundary( Model,Solver,dt,TransientSimulation )
  
  USE DefUtils
  USE SolverUtils
  USE ElementUtils

IMPLICIT NONE
TYPE(Model_t) :: Model
TYPE(Variable_t), POINTER :: MB,Dens,Firn,SE,Depth,Melting,TimeVar
TYPE(Solver_t), POINTER :: Solver
TYPE(ValueList_t), POINTER :: SolverParams
TYPE(Element_t),POINTER :: Element
INTEGER, POINTER :: NodeIndexes(:)

INTEGER :: n,i,j,nb,k,cont,Day,ierr=0,year,nb_surf,nb_vert
REAL(KIND=dp) :: f, g, x, z, deg_pos,accu_ref,a,deg_jour_snow,surimposed_ice_fact,deg_jour_ice,precip_fact
REAL(KIND=dp) :: accu,melt_local,accu_ice,melt,accusnow,accuice,z_precip,grad_accu,deg_jour,dt,T,alt,grad,seuil_precip
REAL(KIND=dp) :: seuil_fonte, dP_fact, dt_scale,Tref,precip_fact_ref,alpha,Tmean,AccuMean,tsurf,T0,time,delta_tsurf,t_simu
logical :: found,GotIt,first_time=.true.,TransientSimulation


save first_time,nb_surf,nb_vert,time

if (first_time) then
first_time=.false.

do i=1,model % NumberOfNodes
if (model % nodes % x(i+1)<model % nodes % x(i)) then
exit
endif
enddo

nb_surf=i
nb_vert=model % NumberOfNodes/i

endif

!SI => VariableGet( Model % Variables, 'Surimposed_ice')
!SI % values (SI % perm(n)) = accu_ice

Firn => VariableGet( Model % Variables, 'Firn')
Dens => VariableGet( Model % Variables, 'Densi')
Depth => VariableGet( Model % Variables, 'Depth')
SE => VariableGet( Model % Variables, 'Surf Enth')
MB => VariableGet( Model % Variables, 'Mass Balance')
Melting => VariableGet( Model % Variables, 'Melting')
T0=GetConstReal(Model % Constants, "T_ref_enthalpy")
TimeVar => VariableGet(Model % Mesh % Variables, 'time' )

t_simu=TimeVar % Values(1)

alt=5000.0 
grad=0.006
seuil_precip=2.0
seuil_fonte=0.0

deg_jour_ice=0.00452
surimposed_ice_fact=0.0
deg_jour_snow=0.00156
grad_accu=0.00040
z_precip=5000.0

alpha=GetConstReal(Model % Constants, "delta_T")
Tmean=GetConstReal(Model % Constants, "Tmean")
AccuMean=GetConstReal(Model % Constants, "Precip")
delta_tsurf=-2.0


DO n=1,model % NumberOfNodes

if (Depth % Values (Depth % perm (n))==0.0) then

z= model % nodes % y(n)
accu_ref=0.0
deg_pos=0.0

DO i=1,365

    T=alpha*sin(2*3.14*i/365)+grad*(alt-z)+Tmean

   
    if (T>seuil_fonte) then
        deg_pos=deg_pos+(T-seuil_fonte)
    endif
    
     if (T<seuil_precip) then
        accu_ref=accu_ref+1.0/floor(1.0*365.25)
    endif
ENDDO

!Get Mass Balance

accu=accu_ref*(AccuMean+(z-z_precip)*grad_accu)

melt_local=deg_pos*deg_jour_snow
accu_ice=min(surimposed_ice_fact*accu,melt_local)

    a=-melt_local-accu_ice*(1.0+1.0/((1.0-0.358/0.917)*1000/358.0))+accu
    if (a>=0.0) then
        deg_jour=deg_jour_snow
    else
        deg_jour=deg_jour_ice-(deg_jour_ice-deg_jour_snow)*(accu-accu_ice*(1.0+1.0/((1.0-0.358/0.917)*1000/358.0)))/melt_local
    endif


MB % values (MB % perm(n)) = (accu_ice+accu-deg_pos*deg_jour)/0.917
Melting % values (Melting % perm(n)) = deg_pos*deg_jour


tsurf=grad*(alt-z)+Tmean+273.15+delta_tsurf


if (mod(floor(t_simu/dt),2)==1) then
tsurf=tsurf+alpha/2.0
else
tsurf=tsurf-alpha/2.0
endif

if (tsurf > 273.15) then
tsurf = 273.15
endif

SE % values (SE % perm(n)) = 3.626*tsurf**2+146.3*tsurf-T0*(146.3+3.626*T0)

Firn % values (Firn % perm(n)) = Firn % values (Firn % perm(n)) + a*dt - Firn % values (Firn % perm(n))*0.3*dt/10.0

if (Firn % values (Firn % perm(n))<0.0) then
	Firn % values (Firn % perm(n)) = 0.0
endif

do i=1,nb_vert

cont=n-(i-1)*nb_surf

if (Firn % values (Firn % perm(n))>1.0) then
Dens % values (Dens % perm(cont))=350.0+Depth % values (Depth % perm(cont))/Firn % values (Firn % perm(n))*(917.0-350.0)
else
Dens % values (Dens % perm(cont))=917.0
endif

if (Dens % values (Dens % perm(cont))>917.0) then
Dens % values (Dens % perm(cont))=917.0
endif


enddo

endif
ENDDO


 
END SUBROUTINE SurfBoundary
